/*
    (C) 1995-2001 AROS - The Amiga Research OS
    $Id: makedir.c,v 1.1.1.1 2005/03/15 16:00:59 laire Exp $
 
    Desc: MakeDir CLI command
    Lang: English
*/

#include <dos/dos.h>
#include <utility/tagitem.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include <stdio.h>

#include "makedir_version.h"

static const char version[] = VERSTAG;

/******************************************************************************
 
    NAME
 
        MakeDir
 
    SYNOPSIS
 
        NAME/M
 
    LOCATION
 
        Workbench:C
 
    FUNCTION
 
        Create new empty directories with specified names.
 
    INPUTS
 
        NAME  --  names of the directories that should be created
 
    RESULT
 
    NOTES
 
        MakeDir does not create an icon for a new directory.
 
    EXAMPLE
 
    BUGS
 
    SEE ALSO
 
    INTERNALS
 
    HISTORY
 
    12 November 2000,  SDuvan  --  Implmented multiple target support,
                                   rewrote to live up to AmigaDOS
				   semantics 
 
******************************************************************************/

#define TEMPLATE "NAME/M,ALL/S"
enum
{
    ARG_NAME = 0,
    ARG_ALL,
    NOOFARGS
};

LONG CreateDirAll(struct DosLibrary *DOSBase, STRPTR name);

int main(void)
{
    struct Library *SysBase=*((struct Library**) 4);
    struct DosLibrary	*DOSBase;
    LONG   error = RETURN_FAIL;

    DOSBase = (struct DosLibrary *)OpenLibrary("dos.library", 37);

    if (DOSBase)
    {
        IPTR           args[NOOFARGS] = { NULL };
        struct RDArgs *rda;

        rda = ReadArgs(TEMPLATE, args, NULL);

        if (rda)
        {
            int      i = 0;
            STRPTR  *name = (STRPTR *)args[ARG_NAME];

            if (!name)
            {
                PutStr("No name given\n");
                error = RETURN_FAIL;
                SetIoErr(0);
            }
            else
            {
                BPTR   lock;

                for (i = 0; name[i]; i++)
                {
                    if (args[ARG_ALL])
                        lock = CreateDirAll(DOSBase, name[i]);
                    else
                        lock = CreateDir(name[i]);

                    /* The AmigaDOS semantics are quite strange here. When it is
                       impossible to create a certain directory, MakeDir goes on
                       to try to create the rest of the specified directories and
                       returns the LAST return value for the operation. */
                    if (lock)
                    {
                        UnLock(lock);
                        error = RETURN_OK;
                    }
                    else
                    {
                        Printf("Cannot create directory %s\n", (ULONG)name[i]);
                        error = RETURN_ERROR;
                    }
                }
            }

            FreeArgs(rda);
        }
        else
            error = RETURN_FAIL;

        if (error != RETURN_OK && IoErr())
            PrintFault(IoErr(), NULL);

        CloseLibrary((struct Library *) DOSBase);
    }

    return error;
}

/* CreateDirAll
 *
 * Walk path from left to right, Lock()ing each element. If locking fails,
 * try CreateDir.
 * This routine is smart enough to try optimize multiple '/'s.
 */

LONG CreateDirAll(struct DosLibrary *DOSBase, STRPTR name)
{
    STRPTR pt = name;
    BOOL first = TRUE;
    BPTR oldcurdir;
    BPTR l, o;
    UBYTE oc = 0;
    int skip = 0;
    UBYTE _fib[sizeof(struct FileInfoBlock) + 3];
    struct FileInfoBlock *fib = (APTR) ((((IPTR) _fib) + 3) & ~3);

    CurrentDir(oldcurdir = CurrentDir(0));

    for (;;)
    {
        UBYTE c = *pt++;

        if (c == ':' || c == '/' || c == '\0')
        {
            if (c == ':')
            {
                skip = 0;

                if (!first)
                {
                    SetIoErr(ERROR_DEVICE_NOT_MOUNTED);
                    break;
                }
                first = FALSE;
                oc = pt[skip];
                pt[skip] = '\0';
                //Printf("Lock \"%s\"\n", (LONG) name);
                l = Lock(name, ACCESS_READ);
            }
            else
            {
                skip = 0;
                if (c == '/')
                {
                    while (pt[skip] == '/')
                    {
                        skip++;
                    }
                }

                oc = pt[skip];
                pt[skip] = '\0';
                //Printf("Lock \"%s\"\n", (LONG) name);
                l = Lock(name, ACCESS_READ);
                if (!l)
                {
                    pt[skip] = oc;
                    skip = *name != '/' && c == '/' ? -1 : 0;
                    oc = pt[skip];
                    pt[skip] = '\0';

                    //Printf("CreateDir \"%s\"\n", (LONG) name);
                    l = name[0] == '/' || name[0] == '\0' ? 0 : CreateDir(name);
                    if (l)
                    {
                        if (!ChangeMode(CHANGE_LOCK, l, ACCESS_READ))
                        {
                            UnLock(l);
                            l = Lock(name, ACCESS_READ);
                        }
                    }
                }
                else
                {
                    LONG res;

                    /* Make sure it's a directory */
                    if (!(res = Examine(l, fib)) || fib->fib_DirEntryType < 0)
                    {
                        UnLock(l);
                        if (res)
                        {
                            SetIoErr(c == '\0' ? ERROR_OBJECT_EXISTS : ERROR_OBJECT_WRONG_TYPE);
                        }
                        break;
                    }
                    pt += skip;
                    skip = 0;
                }
            }

            if (!l)
            {
                break;
            }

            o = CurrentDir(l);
            if (o != oldcurdir)
            {
                UnLock(o);
            }

            pt[skip] = oc;
            name = pt;

            if (c == '\0')
            {
                //Printf("return success\n");
                return CurrentDir(oldcurdir);
            }
        }
    }

    pt[skip] = oc;

    o = CurrentDir(oldcurdir);
    if (o != oldcurdir)
    {
        UnLock(o);
    }

    //Printf("return error\n");
    return 0;
}

